static char *cvs_id =" $Id: xnd_prac.c,v 1.1.1.1 2000/05/28 13:19:00 berar Exp $ ";
/* XND_PRAC.C    this file is used for asymetry convolution */
/* =========================================================== */
/* following is an implemtentation of instrumental asymmetry   */
/* derived from paper : van Laar, Yelon JAC 17 (1984)          */
/* revised paper from : Finger, Cox, Jephcoat JAC 27 (1994)    */
/* ----------------------------------------------------------- */
/* change from s and h to s+h and (s-h)/(s+h), rel 1.21 Jun00  */
/* localy modified using the convolution commutativity         */
/* jfb, jan 21th, 98                                           */
/* split the integral near 0 improves numerical integration    */
/* approximation can be used for shard or broad lines          */
/* ----------------------------------------------------------- */
#define  USE_BRAGG
#define USE_VAR
#define USE_EXPER
#include "xnd_prof.h"
static int   is_min;
static int   l_width;     /* left, right used */   
static int   r_width;        
static int   n_width;     /* center position */
static int   c_width;     /* convol width is steps */
static int   t_width;     /* memory size if realloc */
static float dphi_mini, dphi_infl, absct_2t;
static float as_step;
static float as_norm;
static float s_l, h_l, sh_sum, sh_dif, sh_min, sh_rel;
/* following tables are used to convoluate the profil by asymetry */
static float *prof_tab = NULL; 
static float *d_pt_tab = NULL; 
static float *d_pg_tab = NULL; 
static float *d_pl_tab = NULL; 
static float *d_pa_tab[MAX_ASYM] = {NULL, NULL, NULL, NULL, NULL}; 
static void store_profile PROTO((int step));  /* sample profile */
static void realloc_profile PROTO((int e));
/* effective function during calculation */
static float profil_asym0  PROTO((void));  /* convolution tth < 90 */
static void  d_pil_asym0   PROTO((void));
static float profil_asym3  PROTO((void));  /* convolution tth > 90 */
static void  d_pil_asym3   PROTO((void));
static float profil_asym1  PROTO((void));  /* approx for broad lines */
static void  d_pil_asym1   PROTO((void));
static float profil_asym2  PROTO((void));  /* approx for sharp lines */
static void  d_pil_asym2   PROTO((void));
/* for debug ... */
static float previous_sum=0;
static float ocsum = 0;
static float csum;
static int istep, mstep, ois, oms;
#include <errno.h>
extern int errno;
#include <string.h>
/* =========================================================== */
/* allocate memory, ...          for the asymmetry convolution */
/* ----------------------------------------------------------- */
void  ini_asym_convol (P_(void))
{    int ia;
     char *err_msg = "profile table for asymetry convolution";
     t_width = asym_resol*asym_width;
     if (t_width < 200 ) t_width = 200;
     prof_tab = (float * ) tst_alloc(t_width, sizeof(float), err_msg);
     d_pt_tab = (float * ) tst_alloc(t_width, sizeof(float), err_msg);
     d_pg_tab = (float * ) tst_alloc(t_width, sizeof(float), err_msg);
     d_pl_tab = (float * ) tst_alloc(t_width, sizeof(float), err_msg);
     for ( ia = 0; ia < max_asym ; ia++)  d_pa_tab[ia] = (float * ) 
                 tst_alloc(t_width, sizeof(float), err_msg);
     }
/* =========================================================== */
/* calculation of intrisic high resolution profile before conv */
/* dphi_infl, dphi_mini are difference angles from 2theta      */
/* ----------------------------------------------------------- */
int   def_asym    (P_(void))
{  int l;
   char *method;
   push_err_msg("def_asym(xnd_prac.c)");
   temp = &temp_exper;
   var_init (l_exper->ascnv);
/* s_l =   var_valeur (0);  if ( s_l < 0 ) s_l = 0;
   h_l =   var_valeur (1);  if ( h_l < 0 ) h_l = 0;
   sh_sum = s_l + h_l;
   sh_dif = fabs( h_l - s_l);
   if  ( s_l > h_l ) 
   {  is_min = 0;
      sh_min = 2*s_l;
      } else
   {  is_min = 1;
      sh_min = 2*h_l;
      }
*/
	sh_sum = var_valeur (0);
   sh_rel = fabs(var_valeur (1));
   sh_dif = sh_rel * sh_sum;
	sh_min = sh_sum - sh_dif;
   tst_abort( (sh_sum <EPSILON), "convol asym needs non ZERO sum of values");
   if (test_mode & 0x04 && (cycle == 0) && (minli <1)) 
   {  if (ois || oms) verbose_printf("prev_asym   ------- p_sum %.3g integ %5.3f (is=%d, ms=%d)\n\n", 
              previous_sum, ocsum, ois, oms);
       ocsum = 0;
       previous_sum=0;
       ois = oms =0;
       }
    /* calculates characteritic values  : dt_mini, dt_infl */
    /* assuming their value is constant on the line        */    
    absct_2t=fabs(cotan_2t);
    dphi_infl = (0.5 / RAD) * sh_dif*sh_dif * absct_2t;
    dphi_mini = (0.5 / RAD) * sh_sum*sh_sum * absct_2t;
    if (cotan_2t > 0 )
    {  if ( dphi_infl > DTH ) dphi_infl = DTH;
       if ( dphi_mini > DTH ) dphi_mini = DTH;
       }
    else   
    {  if ( dphi_infl > 180-DTH ) dphi_infl = 180-DTH;
       if ( dphi_mini > 180-DTH ) dphi_mini = 180-DTH;
       }
    /* the normalization of the practically used function */
    as_norm = sh_sum * sqrt( 2/RAD * absct_2t * dphi_mini )
/*            + (sh_min-sh_sum) * sqrt( 2/RAD * absct_2t * dphi_infl ) */
/*            - sh_dif * sqrt( 2/RAD * absct_2t * dphi_infl ) */
	    - dphi_mini + dphi_infl;
    /* defines the method : real convolution or approximation */
    if ( w_asym < asym_rmin * dphi_mini )
    {  method = "instru approx";
	    profil_asym = profil_asym2;
	    d_pil_asym = d_pil_asym2;
       absct_2t /=2*RAD;
	 }
   else if ( dphi_mini < asym_rmax * w_asym )
   {  method = "sample approx";
	   profil_asym = profil_asym1;
	   d_pil_asym = d_pil_asym1;
	   }
   else  /* we have to prepare the real convolution */
   {  method = "convol";
      absct_2t /=2*RAD;
	   if (cotan_2t>0)
      {   profil_asym = profil_asym0;
	      d_pil_asym = d_pil_asym0;
              } else
	  {   profil_asym = profil_asym3;
	      d_pil_asym = d_pil_asym3;
	      }
      /*  store the line profile in a table for further uses */
      as_step = w_asym / asym_resol;
      l = asym_resol*asym_width;
      c_width = 1 + dphi_mini / as_step;
      if ( c_width < 2 * asym_resol )
      {   as_step = 0.5 * dphi_mini / asym_resol ;
          c_width = 1 + dphi_mini / as_step;
          }
      if ( l < 2* c_width ) l= 2*c_width;
      if ( l > t_width ) realloc_profile(  l- t_width);
      l_width = r_width = 0;
      }
    if (test_mode & 0x04 && cycle == 0 && (minli <1)) 
    {  verbose_printf("asym (%2d%2d%2d) %7.3f  wasym=%.4f inf=%5.4f min=%5.4f norm=%.4g --> %s %d, t_width=%d\n",
       H, K, L, DTH, w_asym, dphi_infl, dphi_mini, as_norm, method, c_width, t_width);
       }
    pop_err_msg();
    return 1;  
    }
/* =========================================================== */
static void realloc_profile(P_(int)e)
#ifdef NO_PROTO
int e;
#endif
{  int ia, otw;
   char *err_msg = "reallocating profile tables for convolution";
   push_err_msg("realloc_profile(xnd_prac.c)");
   if ( e < 100 ) e= 100;
   t_width += e;
   otw=t_width*sizeof(float);
   prof_tab = (float * ) realloc(prof_tab, otw);
   tst_abort((prof_tab==NULL), err_msg);
   d_pt_tab = (float * ) realloc(d_pt_tab, otw);
   tst_abort((d_pt_tab==NULL), err_msg);
   d_pg_tab = (float * ) realloc(d_pg_tab, otw);
   tst_abort((d_pg_tab==NULL), err_msg);
   d_pl_tab = (float * ) realloc(d_pg_tab, otw);
   tst_abort((d_pl_tab==NULL), err_msg);
   for ( ia = 0; ia < max_asym ; ia++)
   {  d_pa_tab[ia] = (float * ) realloc(d_pa_tab[ia], otw);
      tst_abort((d_pa_tab[ia]==NULL), err_msg);
      }
   pop_err_msg();
   }  
/* =========================================================== */
/*  just to avoid to much recalculation of the same points     */
static void store_profile(P_(int) step)
#ifdef NO_PROTO
int step;
#endif
{  int ia, i, j, e, s;
   int cew, olw, orw;
/*	int dbg_case; */
   float odt;
   odt = dt;
   olw = l_width;
   orw = r_width;
   if ( l_width == 0 && r_width == 0 ) 
   {  n_width = asym_resol - step;
      s = l_width = step - asym_resol;
      e = r_width = step + 5*asym_resol;    /* 1+asym_width/2 ???*/
      olw = orw = 0;
/*      dbg_case = 0; */
		}
   else  /* part of the profile is soon calculated, trying to keep it */
   {  if ( step + c_width >= r_width ) 
      {  if (step > l_width)       /* corrected in 1.18 */
		   {  s = l_width = step;
			   e = r_width = step + c_width + 1;
				n_width = -s;
/*             dbg_case = -1; */
				}
			else
			{  cew = 2 + r_width - step - c_width;
            if ( cew < c_width ) cew = c_width +1;
            if ( step + cew + n_width < t_width ) 
            {  s = r_width;
               e = r_width = step + cew;
/*             dbg_case = 1; */
               } 
            else 
            {  int otw, ia, sh, fr;
               sh = step + cew - r_width;
             	if ( sh < c_width ) sh = c_width;
            	if ( sh < 50 ) sh = 50;
            	fr = t_width - (r_width - l_width);
            	if ( fr > 0 ) sh -= fr; 
            	s = r_width;
            	n_width += -sh;
            	l_width += sh;
            	e = r_width += c_width;
            	otw = (t_width - sh) * sizeof(float);
            	if ( otw > 0 )
            	{  memmove(prof_tab, prof_tab+sh, otw);
               	memmove(d_pt_tab, d_pt_tab+sh, otw);
               	memmove(d_pg_tab, d_pg_tab+sh, otw);
               	memmove(d_pl_tab, d_pl_tab+sh, otw);
               	for ( ia = 0; ia < max_asym ; ia++)
                  	memmove(d_pa_tab[ia], d_pa_tab[ia]+sh, otw);
               	}
/*            dbg_case = 2; */
               }
            }    
         }  /* if ( step + c_width >= r_width ) */
      if ( step < l_width ) 
      {  if ( step  + c_width < l_width ) 
         {  n_width = -step;
            s = l_width = step - 1;
            e = r_width = step + c_width + 1;
/*            dbg_case = 3; */
            } 
         else 
         {  int otw, ia, sh, fr;
            fr = t_width - (r_width - l_width);
            cew = ( c_width < 20 ) ? 20 : c_width;
            e = l_width;
            s = l_width -= cew;
            n_width += cew;
            if ( fr > cew ) 
            {  sh = t_width - fr;
               }
            else 
            {  sh = t_width - cew;
               r_width = l_width + t_width;
               }
            otw = sh * sizeof(float);
            if ( otw > 0 )
            {  memmove(prof_tab+cew, prof_tab, otw);
               memmove(d_pt_tab+cew, d_pt_tab, otw);
               memmove(d_pg_tab+cew, d_pg_tab, otw);
               memmove(d_pl_tab+cew, d_pl_tab, otw);
               for ( ia = 0; ia < max_asym ; ia++)
                  memmove(d_pa_tab[ia]+cew, d_pa_tab[ia], otw);
               }
/*            dbg_case = 4; */
            }
         }
      }
   if (s - l_width < 0 ) s = l_width; /* corrected in 1.18 */		
   for (i=s, j = s-l_width; i<e; i++, j++)
   {  dt = i * as_step;
      switch (prof_case)
      {  case  1 : 
            prof_tab[j] = gauss_profil() * (*f_asym)(); 
            d_pt_tab[j] = gauss_dprof_dt() * as_coef - w_x * as_deriv; 
            d_pg_tab[j] = gauss_dprof_dg() * as_coef;
            d_pl_tab[j] = 0;
            for ( ia = 0; ia < max_asym ; ia++) d_pa_tab[ia][j] = w_x * as_d[ia];
            break;
         case  2 : 
	         prof_tab[j] = lorentz_profil() * (*f_asym)();  
	         d_pt_tab[j] = lorentz_dprof_dt() * as_coef - w_y * as_deriv;
              d_pl_tab[j] = lorentz_dprof_dl() * as_coef;
              d_pg_tab[j] = 0;
              for ( ia = 0; ia < max_asym ; ia++) d_pa_tab[ia][j] = w_y * as_d[ia];
              break;
           case  3 : 
	      prof_tab[j] = psvoigt_profil() * (*f_asym)(); 
	      d_pt_tab[j] = psvoigt_dprof_dt()* as_coef-(w_x+w_y)*as_deriv;
              d_pg_tab[j] = psvoigt_dprof_dg() * as_coef;
              d_pl_tab[j] = psvoigt_dprof_dl() * as_coef;
              for ( ia = 0; ia < max_asym ; ia++) d_pa_tab[ia][j] = (w_x+w_y) * as_d[ia];
              break;
           case 4 : 
             prof_tab[j] = voigt_profil() * (*f_asym)(); 
	     d_pt_tab[j] = voigt_dprof_dt() * as_coef - w_x * as_deriv;
             d_pg_tab[j] = voigt_dprof_dg() * as_coef;
             d_pl_tab[j] = voigt_dprof_dl() * as_coef;
             for ( ia = 0; ia < max_asym ; ia++) d_pa_tab[ia][j] = w_x * as_d[ia];
	     break;
           case 0 :
             prof_tab[j] = 0; 
	     d_pt_tab[j] = 0;
             d_pg_tab[j] = 0;
             d_pl_tab[j] = 0;
             for ( ia = 0; ia < max_asym ; ia++) d_pa_tab[ia][j] = 0;
             break;
           default : tst_abort(1, "prof_case = %d, switch error, report please!", prof_case);
           }
     }
   dt = odt;     
   }	
/* =========================================================== */
/* does the calculation for a point located at dt from peak    */
/* numerous term of the calculation vanish in normalization    */
/* (2*h_l*cos_2phi) approx by (2*h_l*cos_2t) vanish            */
/* the convolution is evaluated by a numerical summation       */
/* but as the function h_phi diverge at zero, it is integrated */
/* on the first half step before multipliyng by the profile    */
/* then the result seems a reliable approximation without      */
/* using a too thin step inside the line                       */
/* ----------------------------------------------------------- */
static float profil_asym0  (P_(void))
{   int st, step, ostep;
    float dphi, valeur;
    float conv;
    float h_phi;
    push_err_msg("profil_asym0(xnd_prac.c)");
   if (test_mode & 0x04) 
   {    mstep = istep = 0;
        csum = 0;
        }
    ostep = step = dt / as_step;
    if ( step < l_width || step + c_width >= r_width )
        store_profile(step);
    st = step += n_width + 1 ;     
    valeur = 0;
    if ( step < 0 || step >= t_width ) 
    {  verbose_printf(" ostep %d step %d tw %d nw %d, dt=%.4f\n", ostep, step, t_width, n_width, dt);
       verbose_printf(" HKL %3d %3d %3d  %.3f w_asym=%.3f phi_inf=%.3f phi_min=%.3f  l_width=%d c_width=%d r_width=%d\n", H, K, L,  
       DTH, w_asym, dphi_infl, dphi_mini, l_width, c_width, r_width);
       abort();
       }
    if ( step < 0 ) step = 0;
    if ( step < t_width ) 
    do 
    {  dphi = (step-st) * as_step; 
/*if ( (step -st)%400==0) verbose_printf(" step %3d  step-st %3d dphi %f prof_tab %lf\n",
  step, step-st, dphi, prof_tab[step]); */
       if ( dphi < as_step )   
       valeur += (prof_tab[step]+prof_tab[step+1])* sh_min * sqrt((dphi+as_step/2)*absct_2t)/as_step;
       else
       {  h_phi = sqrt(dphi/absct_2t);
          if ( dphi < dphi_infl )  valeur += prof_tab[step] * sh_min / h_phi ;
          else if ( dphi < dphi_mini )
	  {   conv = sh_sum / h_phi - 1.0;  
	      if ( conv <= 0 ) break;  /* take care of rounding in approx */
              valeur += conv * prof_tab[step];
	      }
          else  break;
	  }
if (test_mode & 0x04) 
{   if ( dphi < as_step )   
    {   conv = 2 * sh_min * sqrt((dphi+as_step/2)*absct_2t)/as_step;
	istep++;
	}
    else
    {  if ( dphi < dphi_infl ) 
       {  conv = sh_min / h_phi ;
	  istep++;
	  }
       else if ( dphi < dphi_mini )
       {  conv = sh_sum / h_phi -1.0;  
          mstep++;
	  }
       }
    csum += conv;
/*    if (ois==0 && (dbg <=2)&& H==2 && K==2 && L==0 ) 
    verbose_printf("pro_asym   ------- c_sum %.3g conv %.3g dphi %.4f is %d ms %d\n", 
              csum, conv, dphi, istep, mstep);
*/    } /* test_mode */
       } while (++step < t_width );
    valeur *= as_step / as_norm;
    if (test_mode & 0x04) 
    {    csum *= as_step / as_norm;
         previous_sum +=valeur;
        if (csum > ocsum) 	ocsum = csum;
        if (istep > ois) 	ois = istep;
        if (mstep > oms) 	oms = mstep;
        }
/*    if (( ostep % 100 == 0  && ( dbg <= 2 || ( ostep >= 0 && ostep <= 100 ))) || valeur < 0 || valeur > 10000 )
 verbose_printf(" %.4f  %10.5f  step %d %d lw %d rw %d cw %d nw %d prof_tab[0]=%f\n", 
 dt, valeur, ostep, step, l_width, r_width, c_width, n_width, prof_tab[0]);
*/    pop_err_msg();
    return valeur;
    }
/* =========================================================== */
/* ----------------------------------------------------------- */
static void  d_pil_asym0    (P_(void))
{  int st, step, ia;
   float dphi;
   float wphi, conv, cv;
   float h_phi;
   push_err_msg("d_pil_asym");
   d_p_t = d_p_g= d_p_l= 0;
   step = dt / as_step;
   st = step += n_width + 1 ;
   d_pa_ins[0] = 0;
   d_pa_ins[1] = 0;
   if (step < 0) step = 0;
   if ( step < t_width ) 
   do 
   {  dphi = (step-st) * as_step; 
      if ( dphi < as_step )   
	   {  cv = 2 * sqrt((dphi+as_step/2)*absct_2t)/as_step;
		   conv = sh_min * cv;
	      d_pa_ins[0] += (1 - sh_rel) * cv * prof_tab[step];
/*         d_pa_ins[1] -= sh_sum * cv * prof_tab[step];
*/         }
      else
	   {  h_phi = sqrt(dphi/absct_2t);
         if ( dphi < dphi_infl )
	      {  conv = sh_min / h_phi;
	         d_pa_ins[0] += (1 - sh_rel) * prof_tab[step] / h_phi;
	         d_pa_ins[1] -= sh_sum * prof_tab[step] / h_phi;
            }
		   else if ( dphi < dphi_mini )
	      {  conv = sh_sum / h_phi -1.0;  
	         if ( conv <= 0 ) break;  /* take care of rounding in approx */
	         d_pa_ins[0] += prof_tab[step] / h_phi;
            } else  break;
	      }
      d_p_t += conv * d_pt_tab[step];
      d_p_g += conv * d_pg_tab[step];
      d_p_l += conv * d_pl_tab[step];
      for ( ia = 0; ia < max_asym ; ia++) as_d[ia] += conv * d_pa_tab[ia][step];
      } while (++step < t_width );
   d_p_t  *= as_step / as_norm;
   d_p_g  *= as_step / as_norm;
   d_p_g  *= as_step / as_norm;
   for ( ia = 0; ia < max_asym ; ia++) as_d[ia] *= as_step / as_norm;
   d_pa_ins[0] *= as_step / as_norm;
   d_pa_ins[1] *= as_step / as_norm;
   pop_err_msg();
   }
/* =========================================================== */
/* Line width bigger than asymetry convolution width          */
/* approximation is possible
/* ----------------------------------------------------------- */
static float profil_asym1  (P_(void))
{  int j, ia;
   float dphi, valeur;
   float conv;
   float h_phi;
   float odt;
   push_err_msg("profil_asym1(xnd_prac.c)");
   odt = dt;
   for (j = 0; j<1; j++)
   {  dt = odt + j * as_step;
      switch (prof_case)
      {  case 1 : 
            prof_tab[j] = gauss_profil() * (*f_asym)(); 
            d_pt_tab[j] = gauss_dprof_dt() * as_coef - w_x * as_deriv; 
            d_pg_tab[j] = gauss_dprof_dg() * as_coef;
            d_pl_tab[j] = 0;
            for ( ia = 0; ia < max_asym ; ia++) d_pa_tab[ia][j] = w_x * as_d[ia];
            break;
         case 2 : 
	         prof_tab[j] = lorentz_profil() * (*f_asym)();  
	         d_pt_tab[j] = lorentz_dprof_dt() * as_coef - w_y * as_deriv;
            d_pl_tab[j] = lorentz_dprof_dl() * as_coef;
            d_pg_tab[j] = 0;
            for ( ia = 0; ia < max_asym ; ia++) d_pa_tab[ia][j] = w_y * as_d[ia];
            break;
         case 3 : 
	         prof_tab[j] = psvoigt_profil() * (*f_asym)(); 
	         d_pt_tab[j] = psvoigt_dprof_dt()* as_coef-(w_x+w_y)*as_deriv;
            d_pg_tab[j] = psvoigt_dprof_dg() * as_coef;
            d_pl_tab[j] = psvoigt_dprof_dl() * as_coef;
            for ( ia = 0; ia < max_asym ; ia++) d_pa_tab[ia][j] = (w_x+w_y) * as_d[ia];
            break;
         case 4 : 
            prof_tab[j] = voigt_profil() * (*f_asym)(); 
	         d_pt_tab[j] = voigt_dprof_dt() * as_coef - w_x * as_deriv;
            d_pg_tab[j] = voigt_dprof_dg() * as_coef;
            d_pl_tab[j] = voigt_dprof_dl() * as_coef;
            for ( ia = 0; ia < max_asym ; ia++) d_pa_tab[ia][j] = w_x * as_d[ia];
	         break;
         }
	   } 
   valeur = prof_tab[0] - d_pt_tab[0] * dphi_infl / 3.0;   
	dt = odt;
   pop_err_msg();
   return valeur;
   }
/* =========================================================== */
/* ----------------------------------------------------------- */
static void  d_pil_asym1    (P_(void))
{   int st, step, ia;
    float dphi;
    float wphi, conv;
    float h_phi;
    push_err_msg("d_pil_asym1");
    d_p_t += d_pt_tab[0];
    d_p_g += d_pg_tab[0];
    d_p_l += d_pl_tab[0];
    for ( ia = 0; ia < max_asym ; ia++) as_d[ia] += d_pa_tab[ia][0];
    d_pa_ins[0] = 0;
    d_pa_ins[1] = 0;
    pop_err_msg();
    }
/* =========================================================== */
/* Line width thiner than asymetry convolution width          */
/* approximation is possible                                   */
/* ----------------------------------------------------------- */
static float profil_asym2  (P_(void))
{   float dphi, valeur;
    float conv;
    float h_phi;
    push_err_msg("profil_asym2(xnd_prac.c)");

    valeur = 0;
    dphi = (cotan_2t>0) ? -dt : dt;
    if ( dphi >= 0)
    {  if ( dphi < as_step  )   
          valeur = sh_min * sqrt((dt+as_step/2)*absct_2t)/as_step;
       else
       {   h_phi = sqrt(dphi/absct_2t);
           if ( dphi < dphi_infl )  valeur = sh_min / h_phi ;
           else if ( dphi < dphi_mini )
	   {   conv = sh_sum / h_phi -1.0;  
	       if ( conv <= 0 ) conv=0;  /* take care of rounding in approx */
               valeur = conv;
	       }
           else  valeur = 0;
           }
	}
verbose_printf("pro2 dt %f dphi %f conv %f valeur %f\n", dt, dphi, conv, valeur);
    valeur *= as_step / as_norm;
    pop_err_msg();
    return valeur;
    }
/* =========================================================== */
/* ----------------------------------------------------------- */
static void  d_pil_asym2    (P_(void))
{   int ia;
    float dphi, valeur;
    float conv;
    float h_phi;
    push_err_msg("d_pil_asym");
    d_p_t = d_p_g= d_p_l= 0;
    d_pa_ins[0] = 0;
    d_pa_ins[1] = 0;
    valeur = 0;
    dphi = (cotan_2t>0) ? dt : -dt;
    if ( dphi < as_step )   
    {  conv = 2 * sh_min * sqrt((dphi+as_step/2)*absct_2t)/as_step;
       d_pa_ins[0] += 4 * sqrt((dphi+as_step/2)*absct_2t)/as_step;
       d_pa_ins[1] -= 4 * sh_sum * sqrt((dphi+as_step/2)*absct_2t)/as_step;
       }
    else
    {  h_phi = sqrt(dphi/absct_2t);
       if ( dphi < dphi_infl )
       {  conv = sh_min / h_phi;
          d_pa_ins[0] += 2 / h_phi;
          d_pa_ins[1] -= 2 * sh_sum / h_phi;
          }
       else if ( dphi < dphi_mini )
       {   conv = sh_sum / h_phi -1.0;  
           if ( conv <= 0 ) conv = 0;  /* take care of rounding in approx */
           d_pa_ins[0] += 1 / h_phi;
           }
	}
     d_p_t += conv;
     d_p_g += conv;
     d_p_l += conv;
     for ( ia = 0; ia < max_asym ; ia++) as_d[ia] += conv;
    d_p_t  *= as_step / as_norm;
    d_p_g  *= as_step / as_norm;
    d_p_g  *= as_step / as_norm;
    for ( ia = 0; ia < max_asym ; ia++) as_d[ia] *= as_step / as_norm;
    d_pa_ins[0] *= as_step / as_norm;
    d_pa_ins[1] *= as_step / as_norm;
    pop_err_msg();
    }
/* =========================================================== */
/* same convolution than profil_asym0 but dth > 90            */
/* ----------------------------------------------------------- */
static float profil_asym3  (P_(void))
{   int st, step;
    float dphi, valeur;
    float conv;
    float h_phi;
    push_err_msg("profil_asym3(xnd_prac.c)");

if (test_mode & 0x04) 
{    mstep = istep = 0;
     csum = 0;
     }
    step = dt / as_step;
    st = step += n_width + 1 ;     
    valeur = 0;
   if ( step >= t_width) step = t_width-1;
        if ( step >= 0 ) 
        do 
        {   dphi = -(step-st) * as_step; 
          if ( dphi < as_step )   
             valeur += (prof_tab[step]+prof_tab[step-1]) * sh_min * sqrt((dphi+as_step/2)*absct_2t)/as_step;
          else
	  {  h_phi = sqrt(dphi/absct_2t);
             if ( dphi < dphi_infl ) valeur += prof_tab[step] * sh_min / h_phi ;
             else if ( dphi < dphi_mini ) 
	     {   conv = sh_sum / h_phi -1.0;
                  if ( conv <= 0 ) break;  /* take care of rounding in approx */
                  valeur += conv * prof_tab[step];
		  }
             else break;
	     }
          } while (--step >= 0 );
    valeur *= as_step / as_norm;
    pop_err_msg();
    return valeur;
    }
/* =========================================================== */
/* ----------------------------------------------------------- */
static void  d_pil_asym3    (P_(void))
{   int st, step, ia;
    float dphi;
    float wphi, conv;
    float h_phi;
    push_err_msg("d_pil_asym");
    d_p_t = d_p_g= d_p_l= 0;
    step = dt / as_step;
    st = step += n_width + 1 ;
    d_pa_ins[0] = 0;
    d_pa_ins[1] = 0;
   if ( step >= t_width) step = t_width-1;
        if ( step >= 0 ) 
        do 
       {  dphi = -(step-st) * as_step; 
          if ( dphi < as_step )   
	  {  d_pa_ins[is_min] += 4 * sqrt((dphi+as_step/2)*absct_2t)/as_step;
             conv = 2 * sh_min * sqrt((dphi+as_step/2)*absct_2t)/as_step;
	     }
          else
	  {  h_phi = sqrt(dphi/absct_2t);
             if ( dphi < dphi_infl )
	     {  conv = sh_min / h_phi;
	        d_pa_ins[is_min] += 2 * prof_tab[step] / h_phi;
                }
		else if ( dphi < dphi_mini )
	      {   conv = sh_sum / h_phi -1.0;  
	          if ( conv <= 0 ) break;  /* take care of rounding in approx */
	          d_pa_ins[0] += prof_tab[step] / h_phi;
	          d_pa_ins[1] += prof_tab[step] / h_phi;
                  }
              else  break;
	     }
          d_p_t += conv * d_pt_tab[step];
          d_p_g += conv * d_pg_tab[step];
          d_p_l += conv * d_pl_tab[step];
          for ( ia = 0; ia < max_asym ; ia++) as_d[ia] += conv * d_pa_tab[ia][step];
           } while (--step >= 0 );
    d_p_t  *= as_step / as_norm;
    d_p_g  *= as_step / as_norm;
    d_p_g  *= as_step / as_norm;
    for ( ia = 0; ia < max_asym ; ia++) as_d[ia] *= as_step / as_norm;
    d_pa_ins[0] *= as_step / as_norm;
    d_pa_ins[1] *= as_step / as_norm;
    pop_err_msg();
    }
/* =========================================================== */
/* ----------------------------------------------------------- */
/* ----------------------------------------------------------- */
/* ----------------------------------------------------------- */
/* =========================================================== */


