static char *cvs_id =" $Id: xnd_prps.c,v 1.1.1.1 2000/05/13 13:12:00 berar Exp $ ";
/* XND_PRPS.C   pseudo-voigt profile function */

#define  USE_BRAGG
#include "xnd_prof.h"
/* =========================================================== */
/* pseudo-voigt parameters for voigt profile error < 1E-3   */
/* parameters cf David JAC 19(86) p63-64     */
/* ----------------------------------------------------------- */
#define PV_I     7     /* degree of polynomial expansion */
#define PV_Y     6
#define PV_J     7
#define PV_X     5
static float   pv_i[PV_I+1] = { 0.0, 0.00268, 0.75458, 2.88898,
                -3.85144, -0.55765, 3.03824, -1.27539 };
static float   pv_y[PV_Y+1] = { 1.0, -0.99725, 1.14594, 2.56150,
                -6.52088, 5.82647, -1.91086 };
static float   pv_j[PV_J+1] = { 0.0, 1.35248, 0.41168, -2.18731,
                6.42452, -10.29036, 6.88093, -1.59194 };
static float   pv_x[PV_X+1] = { 1.0, -0.50734, -0.22744, 1.63804,
                -2.28532, 1.31943 };
#ifndef CONSTANTE_PURE
static float   pv_di[PV_I]  = { 0.00268, 0.75458*2, 2.88898*3,
                -3.85144*4, -0.55765*5, 3.03824*6, -1.27539*7 };
static float   pv_dy[PV_Y]  = { -0.99725, 1.14594*2, 2.56150*3,
                -6.52088*4, 5.82647*5, -1.91086*6 };
static float   pv_dj[PV_J]  = {  1.35248, 0.41168*2, -2.18731*3,
                6.42452*4, -10.29036*5, 6.88093*6, -1.59194*7 };
static float   pv_dx[PV_X]  = { -0.50734, -0.22744*2, 1.63804*3,
                -2.28532*4, 1.31943*5 };
#else
static float   pv_di[PV_I]  = { 0.00268, 1.50916, 8.66694,
                -15.40576, -2.78825, 18.22944, -8.92773 };
static float   pv_dy[PV_Y]  = { -0.99725, 2.29188, 7.6845,
                -26.08352, 29.13235, -11.46516 };
static float   pv_dj[PV_J]  = {  1.35248, 0.82336, -6.56193,
                25.69808, -51.45180, 41.28558, -11.14358 };
static float   pv_dx[PV_X]  = { -0.50734, -0.45488, 4.91412,
                -9.14128, 6.59715 };
#endif
/* ----------------------------------------------------------- */
/* pseudovoigt profile function, position derivative        */
/* lorentz_width and 2*gauss_square_width derivatives :     */
/* derivatives used the stored values in w_x and w_y        */
/* ----------------------------------------------------------- */
float psvoigt_profil (P_(void))
{   float u;
    u = ( prof_dt2 = dt*dt) / x_coef;
    if (u > MINFLOAT )
      if ( u < LN_MAXFLOAT) w_x = i_coef * exp ( - u );
      else w_x = 0;
    else w_x = i_coef;   
    w_y = j_coef / (y_coef + prof_dt2);
    u = w_x + w_y;
    if (u<=0) w_x=w_y=u=0;
    return (u);
    }
/* ----------------------------------------------------------- */
float psvoigt_dprof_dt(P_(void))
{   return (2*dt*(w_x/x_coef + w_y/(y_coef+prof_dt2)));
    }
/* ----------------------------------------------------------- */
float psvoigt_dprof_dg (P_(void))
{   float dl,dg;
    dg = w_x * ( di_dg + dx_dg * prof_dt2 );
    dl = w_y * ( dj_dg - dy_dg / (y_coef + prof_dt2));
    return (dl+dg);
    }
/* ----------------------------------------------------------- */
float psvoigt_dprof_dl (P_(void))
{   float dl,dg;
    dg = w_x * ( di_dl + dx_dl * prof_dt2 );
	 dl = w_y * ( dj_dl - dy_dl / (y_coef + prof_dt2));
    return (dl+dg);
    }

/* ----------------------------------------------------------- */
void ini_ps_int (P_(float *) i, P_(float *) di_dg, P_(float *) di_dl,
            P_(float *) p, P_(float *) dp, P_(int) deg_p,
            P_(float *) r, P_(float *) dr_dg, P_(float *) dr_dl)
#ifdef NO_PROTO
float *i, *di_dg, *di_dl, *p, *dp;
int deg_p;
float *r, *dr_dg, *dr_dl;
#endif
{    float d;
    *i = poly_f (r, deg_p, p);
    d = poly_f (r, deg_p-1, dp);
    *di_dg = d * *dr_dg;
    *di_dl = d * *dr_dl;
    }
/* ----------------------------------------------------------- */
void ini_ps_w ( P_(float *) x, P_(float *) dx_dg, P_(float *) dx_dl,
        P_(float *) p, P_(float *) dp, P_(int) deg_p, P_(float *) g,
        P_(double ) k, P_(float *) r, P_(float *) dr_dg, P_(float *) dr_dl)
#ifdef NO_PROTO
float *x, *dx_dg, *dx_dl, *p, *dp;
int deg_p;
float *g;
double k;
float  *r, *dr_dg, *dr_dl;
#endif
{    float d;
    *x = poly_f (r, deg_p, p) * *g / k;
    d      = poly_f (r, deg_p-1, dp) * *g / k;
    *dx_dg = 2 * *x *( d * *dr_dg + *x / *g);
    *dx_dl = 2 * *x *( d * *dr_dl + *x / *g);
    *x *= *x;
    }

/* ----------------------------------------------------------- */
int ini_psvoigt (P_(void))
{   float d,gam;
    float rho,drho_dg,drho_dl;
    float di_g,dy_g,dj_g,dx_g;
    float di_l,dy_l,dj_l,dx_l;
    float dl,dg;
    gam = W_G + W_L;
    rho = W_G / gam;
    d = gam*gam;
    drho_dg =  W_L / d;
    drho_dl = -W_G / d;
    ini_ps_int (&i_coef, &di_g, &di_l, pv_i, pv_di, PV_I,
           &rho, &drho_dg, &drho_dl);
    ini_ps_w (&y_coef, &dy_g, &dy_l, pv_y, pv_dy, PV_Y, &gam,
        (double) 2, &rho, &drho_dg, &drho_dl);
    rho = W_L / gam;
    drho_dg = -W_L / d;
    drho_dl =  W_G / d;
    ini_ps_int (&j_coef, &dj_g, &dj_l, pv_j, pv_dj, PV_J,
           &rho, &drho_dg, &drho_dl);
    ini_ps_w (&x_coef, &dx_g, &dx_l, pv_x, pv_dx, PV_X, &gam,
          (double)M_SQRT4LN2, &rho, &drho_dg, &drho_dl);
    /* i_coef et j_coef sont les intensites de G et L normalises */
    /* on a i_coef + j_coef = 1 (presque, a renormaliser)        */
    /* x_coef et y_coef etaient les FWHM de G et L normalises    */
    /* ils ont ete modifies en coefficients internes pour calcul */
    /* modification de i_coef et j_coef en coef calcul, normalis */
    /* correction derivee et gauss revue : 2 df/d(g2)  :  1/ W_G */
    rho = i_coef + j_coef;
    dl= (di_l * j_coef - dj_l * i_coef)/ rho;
    dg= (di_g * j_coef - dj_g * i_coef)/ rho;
    di_dl = dl / i_coef - 0.5 * dx_l / x_coef;
    di_dg = ( dg / i_coef  - 0.5 * dx_g / x_coef) / W_G;
    i_coef *= M_1_SQRTPI / sqrt(x_coef) /rho;

    dj_dl = -dl / j_coef + 0.5 * dy_l / y_coef;
    dj_dg = (-dg / j_coef  + 0.5 * dy_l / y_coef) / W_G;
    j_coef *= M_1_PI * sqrt(y_coef) /rho;

    dx_dl = dx_l / (x_coef*x_coef);
    dy_dl = dy_l;
    dx_dg = dx_g / (x_coef*x_coef * W_G);
    dy_dg = dy_g / W_G;

/*    if (test_mode & 0x2)
    fprintf (list_fich," i %f j %f r %f x %f y %f\n i %f %f j %f %f\n x %f %f y %f %f\n",
    i_coef,j_coef,rho,x_coef,y_coef,
    di_dg,di_dl,dj_dg,dj_dl,dx_dg,dx_dl,dy_dg,dy_dl);
*/
    i_coef *= I_C;
    j_coef *= I_C;
    if (i_coef+j_coef/y_coef<0.5*minimum) return (0);
    influence = 2 * ( W_G + W_L) ;
    if (i_coef+j_coef/y_coef < minimum ) influence *=0.5;
    prof_case = 3;
   return (1);
    }
/* ----------------------------------------------------------- */
/* ----------------------------------------------------------- */
/* for analytical convolution                                  */
/* ----------------------------------------------------------- */
float a_psvoigt_profil (P_(void))
{   float u;
    u = ( prof_dt2 = dt*dt) / x_coef;
    if (u > MINFLOAT )
      if ( u < LN_MAXFLOAT) w_x = i_coef * exp ( - u );
      else w_x = 0;
    else w_x = i_coef;   
    w_y = j_coef / (y_coef + prof_dt2);
    u = w_x + w_y;
    if (u<=0) w_x=w_y=u=0;
    return (u);
    }
/* ----------------------------------------------------------- */
float a_psvoigt_dprof_dt(P_(void))
{   return (2*dt*(w_x/x_coef + w_y/(y_coef+prof_dt2)));
    }
/* ----------------------------------------------------------- */
float a_psvoigt_dprof_dg (P_(void))
{   float dl,dg;
    dg = w_x * ( di_dg + dx_dg * prof_dt2 );
    dl = w_y * ( dj_dg - dy_dg / (y_coef + prof_dt2));
    return (dl+dg);
    }
/* ----------------------------------------------------------- */
float a_psvoigt_dprof_dl (P_(void))
{   float dl,dg;
    dg = w_x * ( di_dl + dx_dl * prof_dt2 );
	 dl = w_y * ( dj_dl - dy_dl / (y_coef + prof_dt2));
    return (dl+dg);
    }

/* ----------------------------------------------------------- */
